/* verification_code.css */
body {
    background-color: #f4f4f4;
}

.container {
    display: flex;
    justify-content: center;
    align-items: center;
    height: 100vh;
}

.verification-container {
    background-color: white;
    padding: 20px;
    border-radius: 10px;
    box-shadow: 0px 0px 10px rgba(0, 0, 0, 0.1);
    width: 100%;
    max-width: 400px;
}

.verification-title {
    font-size: 24px;
    font-weight: bold;
    text-align: center;
    margin-bottom: 20px;
}

.verification-instruction {
    font-size: 18px;
    text-align: center;
    margin-bottom: 20px;
}

.verification-form {
    display: flex;
    flex-direction: column;
    align-items: center;
}

.verification-label {
    font-size: 16px;
    font-weight: bold;
    margin-bottom: 10px;
    width: 100%;
}

.verification-input {
    font-size: 14px;
    padding: 10px;
    border: 1px solid #ddd;
    border-radius: 5px;
    width: 100%;
    margin-bottom: 20px;
}

.verification-submit {
    font-size: 16px;
    padding: 10px 20px;
    background-color: #007BFF;
    color: white;
    border: none;
    border-radius: 5px;
    cursor: pointer;
    transition: background-color 0.3s ease;
}

.verification-submit:hover {
    background-color: #0056b3;
}
