__author__ = "Adams Pierre David"
__author_email__ = "adamspd.developer@gmail.com"
__author_website__ = "https://adamspierredavid.com/"
__description__ = "Managing appointment scheduling with customizable slots, staff features, and conflict handling."
__package_name__ = "django-appointment"
__url__ = "https://github.com/adamspd/django-appointment"
__package_website__ = "https://django-appt.adamspierredavid.com/"
__version__ = "3.3.6"
__test_version__ = False
